-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Fichero: ordenacion.ads                                                   --
-- Tema:    Fichero de implementacion de procedimientos de ordenacion        --
-- Fecha:   Diciembre de 2007                                                --
-- Versin: 1.0                                                              --
-- Autor:   Ruben Alejandro Escartin Aparicio                                --
-- Email Univerisdad:   543588@unizar.es                                     --
-- Email Personal:  emulao@gmail.com                                         --
-- Descripcion: Especificacin de algoritmos de ordenacin                   --
-- La implementacin de Quicksort, ordenacin por insercin y puede que      --
--  alguna lnea ms, corresponden a los profesores de la asignatura         --
-- a los profesores de la asignatura, Fernando Tricas y Javier Martnez.     --
-- Todas las implementaciones de los algoritmos de Merge y de Ordenacin     --
-- restantes, son originales de Rubn Escartn en Noviembre-Diciembre de 2007--
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
generic
   -- Modulo de algoritmos de ordenacion
   type tpDato is private;
   with function "<=" (a, b: tpDato) return boolean;
   
   ----------------------------------------------------------------------------
   --   ESPECIFICACIONES FUNCIONALES                   
   ----------------------------------------------------------------------------
   -- ordenado ( v ) = creciente( v ) AND permutacion( v )
   -- creciente ( v ) = PT A EN (v'first..v'last-1).v(A)<=v(A+1)
   -- MAAL permutacion ( v ) = PT A EN (permutacion(v)'first..permutacion(v)'last).
   --                 PT B EN (v'first..v'last).
   --                 NUM (permutacion(v)(A)) = NUM (v(B) AND
   --                 permutacion(v)'last-permutacion(v)'first = v'last-v'first
   -- invertido ( v ) = PT A EN (v'first..v'last).
   --                 invertido(v)(A) = 
   --                 v(((A+v'last-v'first-1) MOD v'last)+v'first)
   ----------------------------------------------------------------------------

   package ordenacion is
      

      -- vectores no restringidos de datos de tipo "tpDato"
      type tpVector is array (integer range <>) of tpDato;

   ----------------------------------------------------------------------------
   function busquedaBinariaAnt (v: tpVector; d: tpDato) return integer;
   ----------------------------------------------------------------------------
   -- Pre:  ordenado( v )
   -- Post: devuelve la posicin del primer elemento SUPERIOR!!
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   function busquedaBinariaPost (v: tpVector; d: tpDato) return integer;
   ----------------------------------------------------------------------------
   -- Pre:  ordenado( v )
   -- Post: devuelve la posicin del primer elemento INFERIOR!!
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure swapInvirtiendo(v:in out tpVector;pivote:integer);
   ----------------------------------------------------------------------------
   --pre: v(v'first..medio2)=v1 AND v(medio2+1..v'last)=v2
   --post: v(v'first..pivote-1):=v2 AND v(pivote..last):=v1
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure swapConSwaps(v:in out tpVector;v1l:integer);
   ----------------------------------------------------------------------------
   --pre: v(v'first..medio2)=v1 AND v(medio2+1..v'last)=v2
   --post: v(v'first..pivote-1):=v2 AND v(pivote..last):=v1
   ----------------------------------------------------------------------------  
   
   ----------------------------------------------------------------------------
   procedure oSwap(v:in out tpVector;v1l,pivote:integer);
   ----------------------------------------------------------------------------
   --pre: v(v'first..medio2)=v1 AND v(medio2+1..v'last)=v2
   --post: v(v'first..pivote-1):=v2 AND v(pivote..last):=v1
   ----------------------------------------------------------------------------  
   

   ----------------------------------------------------------------------------
   procedure classicMerge (v1,v2: tpVector;v: out tpVector);
   ----------------------------------------------------------------------------
   -- Pre:  ordenado( v1 ) AND ordenado( v2 )
   -- Post: creciente( v ) AND permutacion( v , v1&v2 )
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure MergeSwap (V : in out Tpvector; v1l :integer);
   ----------------------------------------------------------------------------
   -- Pre: ordenado( v(v'first..primero-1) ) AND ordenado( v(primero..v'last) )
   -- Post: ordenado(v)
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure MergeInvirtiendo (V : in out Tpvector; v1l :integer);
   ----------------------------------------------------------------------------
   -- Pre: ordenado( v(v'first..primero-1) ) AND ordenado( v(primero..v'last) )
   -- Post: ordenado(v)
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure oMerge (V : in out Tpvector; v1l :integer);
   ----------------------------------------------------------------------------
   -- Pre: ordenado( v(v'first..primero-1) ) AND ordenado( v(primero..v'last) )
   -- Post: ordenado(v)
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure Merge (v1, v2:tpvector; V : out Tpvector);
   ----------------------------------------------------------------------------
   -- Pre: ordenado( v1 ) AND ordenado( v2 )
   -- Post: ordenado(v);
   ----------------------------------------------------------------------------
  
   ----------------------------------------------------------------------------
   procedure oMergeVectores (V : in out Tpvector; v1l :integer);
   ----------------------------------------------------------------------------
   -- Pre: ordenado( v(v'first..primero-1) ) AND ordenado( v(primero..v'last) )
   -- Post: ordenado(v)
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure ordenacionPorInsercion (v: in out tpVector);
   ----------------------------------------------------------------------------
   -- Post: ordenado ( V )
   ----------------------------------------------------------------------------

   ----------------------------------------------------------------------------
   procedure quickSort (v: in out tpVector);
   ----------------------------------------------------------------------------
   -- Post: ordenado ( V )
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure ocSort (V : in out Tpvector);
   ----------------------------------------------------------------------------
   -- Post: ordenado(v)
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure mergesort (V : in out Tpvector) ;
   ----------------------------------------------------------------------------
   -- Post: ordenado(v)
   ----------------------------------------------------------------------------
   
   ----------------------------------------------------------------------------
   procedure classicmergesort (V : in out Tpvector) ;
   ----------------------------------------------------------------------------
   -- Post: ordenado(v)
   ----------------------------------------------------------------------------



end ordenacion;
-------------------------------------------------------------------------------
