-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Fichero: principal.adb                                                    --
-- Tema:    Fichero de implementacion de procedimientos de ordenacion        --
-- Fecha:   Diciembre de 2007                                                --
-- Versin: 1.0                                                              --
-- Autor:   Ruben Alejandro Escartin Aparicio                                --
-- Email Univerisdad:   543588@unizar.es                                     --
-- Email Personal:  emulao@gmail.com                                         --
-- Descripcion: Programa principal para evaluar algoritmos                   --
-- La implementacin de Quicksort, ordenacin por insercin y puede que      --
--  alguna lnea ms, corresponden a los profesores de la asignatura         --
-- Metodologa de la Programacin del CPS, Fernando Tricas y Javier Martnez.--
-- Todas las implementaciones de los algoritmos de Merge y de Ordenacin     --
-- restantes, son originales de Rubn Escartn en Noviembre-Diciembre de 2007--
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
with ada.text_io, ada.integer_text_io, ada.float_text_io,
ada.numerics.Discrete_Random, ada.calendar, ordenacion;
use Ada.Text_Io, Ada.Integer_Text_Io, Ada.Float_Text_Io, Ada.Calendar;


--------------------------------------------------------------------------------
procedure principal is
--------------------------------------------------------------------------------
-- Solicita que se elija la operacin a realizar y el numero de elementos
-- tratados.
-- Devuelve el tiempo medio de ejecucin tras MAXIMOREPETICIONES repeticiones o
-- SEGUNDOSESPERA segundos, lo que ocurra antes.
--------------------------------------------------------------------------------

subtype intervalo is positive range 1..positive'last;
package aleatorio is new ada.Numerics.Discrete_Random(intervalo);
package operar is new ordenacion(integer,"<=");

use  aleatorio, operar;

  ANCHOPANTALLA: constant := 80;
  SEGUNDOSESPERA: constant := 10;
  MAXIMOREPETICIONES: constant := 500;

      ----------------------------------------------------------------------------
      procedure pintaBlancos(n:integer) is
      ----------------------------------------------------------------------------
      -- Pre:  n>=0
      -- Post: dibuja por pantalla "m" espacios en blanco
      ----------------------------------------------------------------------------
      begin
         for i in 1..n loop
                put(' ');
         end loop;
      end pintaBlancos;
      ----------------------------------------------------------------------------
       procedure pintaVector(v:tpVector) is
      ----------------------------------------------------------------------------
      -- Post: dibuja por pantalla el vector v
      ----------------------------------------------------------------------------
      begin
         for i in v'range loop
                put(v(i),0);
         end loop;
         new_line;
      end pintaVector;
      ----------------------------------------------------------------------------
      function numCifras(n:Integer) return integer is
      ----------------------------------------------------------------------------
      -- Pre:  n>=0
      -- Post: numCifras= k+1.10^(k+1)>n>=(10^k)-1
      ----------------------------------------------------------------------------
                 k: integer:=0;
      begin
         while not ((10**(K+1))>n) loop
             k:=k+1;
                 end loop;
         return k+1;
      end numCifras;
      ----------------------------------------------------------------------------
      procedure menu ( algoritmo, nMin, nMax, nStep : out integer) is
      ------------------------------------------------------------------------
      -- Post: algoritmo>=0 and n>= 0
      -- Muestra los algoritmos entre los que elegir.
      ------------------------------------------------------------------------

      -- Variables:

      begin
         new_line;
         put(" Elije que algoritmo cronometrar: ");new_line;
         Put(" swap invirtiendo = 1, swap con swaps = 2");New_Line;
         Put(" swap optimo = 3");New_Line;
         put(" merge invirtiendo = 4");new_line;
         put(" merge swap = 5");new_line;
         put(" merge optimo = 6, merge clasico =7");new_line;
         put(" merge binario = 8, merge con vector auxiliar=9");new_line;
         put(" Elija 1, 2, 3, 4, 5, 6, 7, 8 o 9 (0 para salir): ");
         get(algoritmo);skip_line;new_line;
         if algoritmo<0 or algoritmo>9 then
            put(" Escriba un numero comprendido entre el 1 y el 9.");new_line;
         elsif (algoritmo/=0) then
            Put(" Numero minimo de elementos a tratar: "); Get(Nmin);New_Line;
            nMax:=0;
            while nMax<nMin loop
               Put(" Numero maximo de elementos a tratar: "); Get(Nmax);New_Line;
            end loop;
            if nMin=nMax then nStep:=1;
            else
               nStep:=0;
               while nStep=0 loop
               Put(" Paso entre el numero de elementos a tratar: "); Get(nStep);New_Line;
               end loop;
            end if;
         end if;
      end menu;
      ------------------------------------------------------------------------
      procedure creaVector(T: in out tpVector) is
      ------------------------------------------------------------------------
      -- Pre: cierto
      -- Post: Inserta valores aleatorios en una tabla para cada uno
      --       de sus indices
      ------------------------------------------------------------------------

      -- Variables:
         g:generator;
         d:integer;
      begin

         reset(g);
         for i in T'Range loop
               d:=random(g);
               T(i):=d;
         End loop;

      end creaVector;

      ------------------------------------------------------------------------
      procedure medirTiempoAlgoritmo(algoritmo, n, nMax: integer) is
      ------------------------------------------------------------------------
      -- Pre: tipoOperaci-n>0 and tipoOperaci-n<7 and n>0
      -- Post: Calcula el tiempo necesario para realizar la operaci-n deseada.
      --       En los casos de bsqueda, al ser los tiempos muy reducidos,se
      --       realizan varias veces para hacer posteriormente una media y
      --       obtener as un valor ms fiable..
      ------------------------------------------------------------------------
              procedure CalcularTiempo(n, nMax, repeticiones: integer; hora: float) is
              ----------------------------------------------------------------
              -- Pre: tipoOperaci-n>0 and tipoOperaci-n<7 and hora>0
              -- Post: Calcula el tiempo en las unidades deseadas dependiendo
              --       del tipo de operaci-n y lo escribe por pantalla
              ----------------------------------------------------------------

              -- Variables:
                 tiempo:float;
              begin
                 pintaBlancos(numCifras(nMax)+2-numCifras(n)-((numCifras(nMax)-numCifras(N))/2+1));
                 Put(n,numCifras(n));
                 pintaBlancos((numCifras(nMax)-numCifras(n))/2+1);
                 Put("|");
                 pintablancos((12-numCifras(repeticiones))/2);
                 put(repeticiones,0);
                 pintablancos((12-numCifras(repeticiones))-(12-numCifras(repeticiones))/2);
                 Put("|");
                 tiempo:=hora/float(repeticiones);--Divide por repeticiones
                                                  --para hacer la media
                 if tiempo<0.001 then
                      pintablancos((ANCHOPANTALLA-numCifras(nMax)-33)/2);
                      put((tiempo*1000000.0),0,2,0);
                      put(" microsegundos");new_line;
                 elsif tiempo<1.0 then
                         pintablancos((ANCHOPANTALLA-numCifras(nMax)-32)/2);
                         put((tiempo*1000.0),0,2,0);
                         put(" milisegundos");new_line;
                 else
                         pintablancos((ANCHOPANTALLA-numCifras(nMax)-28)/2);
                         put(tiempo,0,2,0);
                         put(" segundos");new_line;
                 end if;
              end CalcularTiempo;
              ----------------------------------------------------------------

         -- Variables:
         repeticiones:integer:=0;
         hora: float:=0.0;
         hora1,hora2:time;
         v: tpVector(1..n);
         vAux: tpVector(1..n);
         begin

         if algoritmo = 1 then--swapInvirtiendo recursivo (caso medio)----LA VENTAJA SOLO SE NOTA A PARTIR DE 50000 elementos
             while hora<float(SEGUNDOSESPERA) and repeticiones<MAXIMOREPETICIONES loop
               creaVector(v);
               quickSort(v(v'first..n/2));
               quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               swapInvirtiendo(v,v'first+v'last-N/2);
               hora2:=Clock;-- Coge el tiempo de final
               --pintaVector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;
          elsif algoritmo = 2 then--swapConSwaps recursivo (caso medio) VA MAL (por poco)
             while Hora<Float(Segundosespera) and Repeticiones<Maximorepeticiones loop
               creaVector(v);
                quickSort(v(v'first..n/2));
                quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               swapConSwaps(v,N/2);
               hora2:=Clock;-- Coge el tiempo de final
               --pintaVector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;
         elsif algoritmo = 3 then--oSwap recursivo (caso medio)
             while hora<float(SEGUNDOSESPERA) and repeticiones<MAXIMOREPETICIONES loop
               creaVector(v);
               quickSort(v(v'first..n/2));
               quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               oSwap(v,N/2,v'first+v'last-N/2);
               hora2:=Clock;-- Coge el tiempo de final
               --pintaVector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;
          elsif algoritmo = 4 then--mergeInvirtiendo (caso medio)
             while Hora<Float(Segundosespera) and Repeticiones<Maximorepeticiones loop
               creaVector(v);
               quickSort(v(v'first..n/2));
               quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               mergeInvirtiendo(v,N/2);
               hora2:=Clock;-- Coge el tiempo de final
               --pintaVector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;

          elsif algoritmo = 5 then--mergeSwaps recursivo (caso medio)
             while Hora<Float(Segundosespera) and Repeticiones<Maximorepeticiones loop
               creaVector(v);
               quickSort(v(v'first..n/2));
               quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               mergeSwap(v,n/2);
               hora2:=Clock;-- Coge el tiempo de final
               --pintaVector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;
          elsif algoritmo = 6 then--oMerge recursivo (caso medio)
            while Hora<Float(Segundosespera) and Repeticiones<Maximorepeticiones loop
               creaVector(v);
               quickSort(v(v'first..n/2));
               quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               oMerge(v,n/2);
               hora2:=Clock;-- Coge el tiempo de final
               --pintaVector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;
            elsif algoritmo = 7 then--classicMerge (caso medio)
             while Hora<Float(Segundosespera) and Repeticiones<Maximorepeticiones loop
               creaVector(v);
               quickSort(v(v'first..n/2));
               quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               classicMerge(v(v'first..n/2),v(n/2+1..v'last),vAux);
               v:=vAux;
               hora2:=Clock;                   -- Coge el tiempo de final
               --pintavector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;
	   elsif algoritmo = 8 then--merge binario (caso medio)
             while Hora<Float(Segundosespera) and Repeticiones<Maximorepeticiones loop
               creaVector(v);
               quickSort(v(v'first..n/2));
               quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               merge(v(v'first..n/2),v(n/2+1..v'last),vAux);
               v:=vAux; 
               hora2:=Clock;                   -- Coge el tiempo de final
               --pintavector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;
         elsif algoritmo = 9 then--merge vectores (caso medio)
             while Hora<Float(Segundosespera) and Repeticiones<Maximorepeticiones loop
               creaVector(v);
               quickSort(v(v'first..n/2));
               quickSort(v(n/2+1..n));
               --pintavector(v);
               hora1:= Clock;                  -- Coge el tiempo de inicio
               oMergeVectores(v,n/2);
               hora2:=Clock;                   -- Coge el tiempo de final
               --pintavector(v);
               repeticiones:=repeticiones+1;
               hora:=hora+float(hora2 - hora1);-- Tiempo empleado
            end loop;
         end if;

         CalcularTiempo(n,nMax,repeticiones,hora);

      end medirTiempoAlgoritmo;
      ------------------------------------------------------------------------

  -- Variables:
     algoritmo : integer;
     n,nMin,nMax,nStep:integer;

begin
  menu(algoritmo,nMin,nMax,nStep);
  while not (algoritmo=0) loop
      if not ((algoritmo<0) or (algoritmo>9)) then
         new_line;
         pintablancos(24);
        Put("ERROR: 2.00 microsegundos"); 
        new_line(2);
        N:=Nmin;
        pintaBlancos((numCifras(nMax)+1)-((numCifras(nMax)+1)/2));
        Put('N');
        pintaBlancos((numCifras(nMax)+1)/2);
        Put("|Repeticiones|");
        pintaBlancos((ANCHOPANTALLA-(56+(numCifras(nMax))))/2);
        put("TIEMPO MEDIDO EXPERIMENTALMENTE (media)");
        New_Line;
        for I in 1..Anchopantalla loop
           Put('_');
        end loop;
        while N<=Nmax loop
           Medirtiempoalgoritmo(algoritmo,n,nMax);
           n:=n+nStep;
        end loop;
     end if;
     menu(algoritmo,nMin,nMax,nStep);
 end loop;

end principal;
--------------------------------------------------------------------------------                                                     